@@ -8,10 +8,10 @@ from .exceptions import (FBException, FBHTTPException, FBJSONException,
     FBConnectionException)
 
 BASE_GRAPH_URL = "https://graph.facebook.com"
-BASE_FQL_URL = "https://graph.facebook.com/fql?"
 BATCH_QUERY_LIMIT = 50
 TIMEOUT = 60.0
 REQUESTS_CONFIG = {'max_retries': 2}
+VERIFY_SSL = False
 
 log = logging.getLogger(__name__)
 
@@ -23,53 +23,34 @@ def GetRequestFactory(relative_url, **params):
 
     return params
 
-def _issue_request(method, relative_url, **kwargs):
-    """
-    Generic method for making requests to the Graph API and deserializing
-    the response. Here we aggregate all general error-handling & exception
-    catching/raising.
-
-    Returns: deserialized JSON as native Python data structures.
-    """
-    url = BASE_GRAPH_URL + ('/%s' % relative_url)
-    if 'timeout' not in kwargs:
-        kwargs['timeout'] = TIMEOUT
-
-    try:
-        r = requests.request(method, url, config=REQUESTS_CONFIG, **kwargs)
-    except (SSLError, Timeout) as e:
-        raise FBConnectionException(e.message)
-
-    if r.status_code != requests.codes.ok:
-        raise FBHTTPException(r.status_code, r.text)
-
-    try:
-        data = json.loads(r.text)
-    except ValueError as e:
-        log.warn("Error decoding JSON: {0}. JSON={1}".format(e.message, r.text))
-        raise FBJSONException("%s (%s)" % (e.message, r.text))
-
-    return data
-
 #TODO: PUT, DELETE request factories
 
 class FBGraph(object):
-    def __init__(self, access_token='', timeout=TIMEOUT):
+    def __init__(self, access_token='', timeout=TIMEOUT,
+                 pre_hook=None, post_hook=None):
+        """
+        :param access_token: A valid Facebook access token.
+        :param timeout: Per-request HTTP timeout. Default is 60.0 seconds.
+        :param pre_hook: Callable that will run before a request is made.
+        :param post_hook: Callable that will run after a request is made.
+        :return: FBGraph instance.
+        """
 
         self._access_token = access_token
         self._response_fmt = 'json'
         self._timeout = timeout
+        self.pre_hook = pre_hook
+        self.post_hook = post_hook
 
     def get(self, relative_url, params=None):
         """
         Query facebook's graph api at relative_url with
         query string parameters params, where params is a python dict.
         """
-        data = {}
         params = params or {}
         params['access_token'] = self._access_token
 
-        data = _issue_request('get', relative_url, params=params,
+        data = self._issue_request('get', relative_url, params=params,
             timeout=self._timeout)
 
         return data
@@ -108,8 +89,8 @@ class FBGraph(object):
         payload = payload or {}
         payload['access_token'] = self._access_token
         
-        data = _issue_request('post', relative_url, data=payload, files=files,
-            timeout=self._timeout)
+        data = self._issue_request('post', relative_url, data=payload,
+            files=files, timeout=self._timeout)
 
         return data
 
@@ -138,13 +119,13 @@ class FBGraph(object):
         contain the deserialized Facebook error response.
         See: https://developers.facebook.com/docs/reference/api/batch/
         """
-        data = []
         payload = {
             'batch': json.dumps(batch),
             'access_token': self._access_token
         }
 
-        data = _issue_request('post', '', data=payload, timeout=self._timeout)
+        data = self._issue_request('post', '', data=payload,
+            timeout=self._timeout)
         
         # deserialize the body of each batch response, need to make sure it
         # is deserializable, thanks to this bug:
@@ -160,28 +141,47 @@ class FBGraph(object):
         
         return data
 
+    def _issue_request(self, method, relative_url, **kwargs):
+        """
+        Generic method for making requests to the Graph API and deserializing
+        the response. Here we aggregate all general error-handling & exception
+        catching/raising.
 
-class FBQuery(object):
-    def __init__(self, access_token, timeout=TIMEOUT):
-        self._access_token = access_token
-        # currrently only support json response format
-        self._response_fmt = 'json'
-        self._timeout = timeout
+        Returns: deserialized JSON as native Python data structures.
+        """
+        url = BASE_GRAPH_URL + ('/%s' % relative_url)
+        kwargs['timeout'] = kwargs.get('timeout', TIMEOUT)
+        kwargs['verify'] = kwargs.get('verify', VERIFY_SSL)
 
-    def query(self, fql_str):
-        params = {
-            'q': fql_str,
-            'access_token' : self._access_token,
-            'format': self._response_fmt
-        }
+        self._exec_hook('pre')
+        try:
+            r = requests.request(method, url, config=REQUESTS_CONFIG, **kwargs)
+        except (SSLError, Timeout) as e:
+            raise FBConnectionException(e.message)
+        self._exec_hook('post')
 
-        start_time = time.time()
+        if r.status_code != requests.codes.ok:
+            raise FBHTTPException(r.status_code, r.text)
 
-        data = _issue_request('get', 'fql', params=params,
-            timeout=self._timeout)
-
-        stop_time = time.time()
-        duration = stop_time - start_time
-        data['query_time'] = duration
+        try:
+            data = json.loads(r.text)
+        except ValueError as e:
+            log.warn("Error decoding JSON: {0}. JSON={1}".format(e.message, r.text))
+            raise FBJSONException("%s (%s)" % (e.message, r.text))
 
         return data
+
+    def _exec_hook(self, hook_type):
+        hook = None
+
+        if hook_type == 'pre':
+            hook = self.pre_hook
+        elif hook_type == 'post':
+            hook = self.post_hook
+
+        if callable(hook):
+            try:
+                hook()
+            except Exception as e:
+                log.warning('Warning: pre_hook raised %s', e.message)
+
